from cc_connector_cli.connector_cli import run_connector
from red_connector_sshfs.sshfs import Sshfs
from red_connector_sshfs.version import VERSION


def main():
    run_connector(Sshfs, version=VERSION)


if __name__ == '__main__':
    main()



import os
import subprocess
import tempfile
from shutil import which

import jsonschema

sshfs_access_schema = {
    'type': 'object',
    'properties': {
        'host': {'type': 'string'},
        'port': {'type': 'integer'},
        'username': {'type': 'string'},
        'password': {'type': 'string'},
        'dirName': {'type': 'string'},
    },
    'additionalProperties': False,
    'required': ['host', 'username', 'dirName']
}


DEFAULT_PORT = 22
FUSERMOUNT_EXECUTABLES = ['fusermount3', 'fusermount']
SSHFS_EXECUTABLES = ['sshfs']


def _create_password_command(username, host, port, local_path, remote_path, configfile_path):
    """
    Creates a command as string list, that can be executed to mount the <remote_path> to <local_path>, using the
    provided information.
    echo '<password>' | sshfs <username>@<host>:<remote_path> <local_path> -o password_stdin -p <port>
    """
    sshfs_executable, _ = _find_executables()
    remote_connection = '{username}@{host}:{remote_path}'.format(username=username, host=host, remote_path=remote_path)
    return [
        sshfs_executable, remote_connection, local_path,
        '-o', 'password_stdin',
        '-F', configfile_path,
        '-p', str(port)
    ]


def _find_executables():
    sshfs_executable = None
    for executable in SSHFS_EXECUTABLES:
        if which(executable):
            sshfs_executable = executable
            break
    if not sshfs_executable:
        raise Exception('One of the following executables must be present in PATH: {}'.format(
            SSHFS_EXECUTABLES
        ))

    fusermount_executable = None
    for executable in FUSERMOUNT_EXECUTABLES:
        if which(executable):
            fusermount_executable = executable
            break
    if not fusermount_executable:
        raise Exception('One of the following executables must be present in PATH: {}'.format(
            FUSERMOUNT_EXECUTABLES
        ))

    return sshfs_executable, fusermount_executable


class Sshfs:
    @staticmethod
    def receive_directory(access, internal, listing):
        """
        Mounts a directory.

        :param access: A dictionary containing access information. Has the following keys
                       - 'host': The host to connect to
                       - 'username': A username that is used to perform authentication
                       - 'password': A password that is used to perform authentication
                       - 'dirName': The name of the directory to fetch
                       - 'port': The port to connect (optional)
        :param internal: A dictionary containing information about where to mount the directory content
        :param listing: Listing of subfiles and subdirectories which are contained by the directory given in access.
                        Is ignored by this connector.
        :raise Exception: If neither privateKey nor password is given in the access dictionary
        """
        host = access['host']
        username = access['username']
        remote_path = access['dirName']
        local_path = internal['path']

        port = access.get('port', DEFAULT_PORT)
        password = access.get('password')
        private_key = access.get('privateKey')

        if password is not None:
            with tempfile.NamedTemporaryFile('w') as temp_configfile:
                temp_configfile.write('StrictHostKeyChecking=no')
                temp_configfile.flush()
                command = _create_password_command(username, host, port, local_path, remote_path, temp_configfile.name)
                command = ' '.join(command)

                os.mkdir(local_path)

                process_result = subprocess.run(command,
                                                input=password.encode('utf-8'),
                                                stderr=subprocess.PIPE,
                                                shell=True)

                if process_result.returncode != 0:
                    raise Exception('Could not mount "{}" from "{}@{}:{}" using password and "sshfs":\n{}'
                                    .format(local_path, username, host, remote_path, process_result.stderr.decode('utf-8')))
        elif private_key is not None:
            raise NotImplementedError()
        else:
            raise Exception('At least "password" or "privateKey" must be present.')

    @staticmethod
    def receive_directory_validate(access):
        try:
            jsonschema.validate(access, sshfs_access_schema)
        except jsonschema.ValidationError as e:
            if e.context:
                raise Exception(e.context)
            else:
                raise Exception(str(e))
        if ('password' not in access) and ('privateKey' not in access):
            raise Exception('At least "password" or "privateKey" must be present.')

        _ = _find_executables()

    @staticmethod
    def receive_directory_cleanup(internal):
        """
        Unmounts and removes the directory given in internal.

        :param internal: A dictionary containing information about where to unmount a directory.
        """
        path = internal['path']

        _, fusermount_executable = _find_executables()

        process_result = subprocess.run([fusermount_executable, '-u', path], stderr=subprocess.PIPE)
        if process_result.returncode == 0:
            os.rmdir(path)
        else:
            raise Exception('Cleanup failed. Could not unmount "{}" with "{} -u":\n{}'.format(
                fusermount_executable, path, process_result.stderr
            ))



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['red_connector_sshfs']

package_data = \
{'': ['*']}

install_requires = \
['cc-connector-cli>=0.3.0,<0.4.0', 'jsonschema>=2.6,<3.0']

entry_points = \
{'console_scripts': ['red-connector-sshfs = red_connector_sshfs.main:main']}

setup_kwargs = {
    'name': 'red-connector-sshfs',
    'version': '0.2',
    'description': 'RED Connector SSHFS is part of the Curious Containers project.',
    'long_description': '# RED Connector SSHFS\n\nRED Connector SSHFS is part of the Curious Containers project.\n\nFor more information please refer to the Curious Containers [documentation](https://www.curious-containers.cc/).\n\n## Acknowledgements\n\nThe Curious Containers software is developed at [CBMI](https://cbmi.htw-berlin.de/) (HTW Berlin - University of Applied Sciences). The work is supported by the German Federal Ministry of Economic Affairs and Energy (ZIM project BeCRF, grant number KF3470401BZ4), the German Federal Ministry of Education and Research (project deep.TEACHING, grant number 01IS17056 and project deep.HEALTH, grant number 13FH770IX6) and HTW Berlin Booster.\n',
    'author': 'Bruno Schilling',
    'author_email': 's0555131@htw-berlin.de',
    'url': 'https://curious-containers.github.io/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.4,<4.0',
}


setup(**setup_kwargs)
